@@ -1,5 +1,6 @@
+import warnings
 from dataclasses import dataclass, field
-from typing import Any, Callable, List, Optional, Dict, Union, Type
+from typing import Any, Callable, List, Optional, Dict, Union, Type, Literal
 import re
 import numpy as np
 import numpy.typing as npt
@@ -38,10 +39,12 @@ class PreprocessResponse:
     sample_ids: Optional[Union[List[str], List[int]]] = None
     state: Optional[DataStateType] = None
     sample_id_type: Optional[Union[Type[str], Type[int]]] = None
+    sample_ids_to_instance_mappings: Optional[Dict[str, List[str]]] = None # in use only for element instance
+    instance_to_sample_ids_mappings: Optional[Dict[str, str]] = None # in use only for element instance
 
     def __post_init__(self) -> None:
-        def is_valid_string(s: str) -> bool:
-            return bool(re.match(r'^[A-Za-z0-9_]+$', s))
+        assert self.sample_ids_to_instance_mappings is None, f"Keep sample_ids_to_instance_mappings None when initializing PreprocessResponse"
+        assert self.instance_to_sample_ids_mappings is None, f"Keep instance_to_sample_ids_mappings None when initializing PreprocessResponse"
 
         if self.length is not None and self.sample_ids is None:
             self.sample_ids = [i for i in range(self.length)]
@@ -53,10 +56,20 @@ class PreprocessResponse:
             if self.sample_id_type == str:
                 for sample_id in self.sample_ids:
                     assert isinstance(sample_id, str), f"Sample id should be of type str. Got: {type(sample_id)}"
-                    if not is_valid_string(sample_id):
-                        raise Exception(f"Sample id should contain only letters (A-Z, a-z), numbers or '_'. Got: {sample_id}")
         else:
-            raise Exception("length is deprecated.")
+            raise Exception("length is deprecated, please use sample_ids instead.")
+
+        if self.state is None:
+            from code_loader.inner_leap_binder.leapbinder_decorators import store_warning_by_param
+            store_warning_by_param(
+                param_name="PreprocessResponse.state",
+                user_func_name="tensorleap_preprocess",
+                default_value=str("specific order"),
+                link_to_docs="https://docs.tensorleap.ai/tensorleap-integration/writing-integration-code/preprocess-function",
+            )
+
+        else:
+            assert isinstance(self.state, DataStateType), f"PreprocessResponse.state must be of type {DataStateType.__name__} but got {type(self.state)}"
 
     def __hash__(self) -> int:
         return id(self)
@@ -65,8 +78,15 @@ class PreprocessResponse:
         assert self.sample_ids is not None
         return len(self.sample_ids)
 
+@dataclass
+class ElementInstance:
+    name: str
+    mask: npt.NDArray[np.float32]
 
 SectionCallableInterface = Callable[[Union[int, str], PreprocessResponse], npt.NDArray[np.float32]]
+InstanceCallableInterface = Callable[[Union[int, str], PreprocessResponse, int], Optional[ElementInstance]]
+InstanceLengthCallableInterface = Callable[[Union[int, str], PreprocessResponse], int]
+
 
 MetadataSectionCallableInterface = Union[
     Callable[[Union[int, str], PreprocessResponse], int],
@@ -188,6 +208,10 @@ class InputHandler(DatasetBaseHandler):
     shape: Optional[List[int]] = None
     channel_dim: Optional[int] = -1
 
+@dataclass
+class ElementInstanceMasksHandler:
+    name: str
+    function: InstanceCallableInterface
 
 @dataclass
 class GroundTruthHandler(DatasetBaseHandler):
@@ -199,13 +223,20 @@ class MetadataHandler:
     name: str
     function: MetadataSectionCallableInterface
     metadata_type: Optional[Union[DatasetMetadataType, Dict[str, DatasetMetadataType]]] = None
+    use_for_semantic_ls: Optional[bool] = None
 
 
+@dataclass
+class CustomLatentSpaceHandler:
+    function: SectionCallableInterface
+    name: str = 'custom_latent_space'
+
 @dataclass
 class PredictionTypeHandler:
     name: str
     labels: List[str]
-    channel_dim: int
+    channel_dim: Union[int, Literal["tl_default_value"]]= "tl_default_value"
+
 
 
 @dataclass
@@ -223,12 +254,14 @@ class DatasetIntegrationSetup:
     unlabeled_data_preprocess: Optional[UnlabeledDataPreprocessHandler] = None
     visualizers: List[VisualizerHandler] = field(default_factory=list)
     inputs: List[InputHandler] = field(default_factory=list)
+    instance_masks: List[ElementInstanceMasksHandler] = field(default_factory=list)
     ground_truths: List[GroundTruthHandler] = field(default_factory=list)
     metadata: List[MetadataHandler] = field(default_factory=list)
     prediction_types: List[PredictionTypeHandler] = field(default_factory=list)
     custom_loss_handlers: List[CustomLossHandler] = field(default_factory=list)
     metrics: List[MetricHandler] = field(default_factory=list)
     custom_layers: Dict[str, CustomLayerHandler] = field(default_factory=dict)
+    custom_latent_space: Optional[CustomLatentSpaceHandler] = None
 
 
 @dataclass
@@ -239,3 +272,6 @@ class DatasetSample:
     metadata_is_none: Dict[str, bool]
     index: Union[int, str]
     state: DataStateEnum
+    custom_latent_space: Optional[npt.NDArray[np.float32]] = None
+    instance_masks: Optional[Dict[str, ElementInstance]] = None
+
